package com.hzya.frame.plugin.lets.util;

import com.hzya.frame.plugin.lets.dao.*;
import com.hzya.frame.plugin.lets.entity.*;
import com.hzya.frame.plugin.lets.plugin.sales.SoSaleOutPluginInitializerToC;
import com.hzya.frame.plugin.lets.resultvo.CacheTocMapVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 档案类缓存，TOC销售订单转换
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：BasicArchivesCacheUtil
 * @Date：2024/7/31 11:27
 * @Filename：BasicArchivesCacheUtil
 */
@Component
public class TocOrderBasicArchivesCacheUtil {

    Logger logger = LoggerFactory.getLogger(TocOrderBasicArchivesCacheUtil.class);

    @Autowired
    private IBdCorpDao iBdCorpDao;

    @Autowired
    private IBdStordocDao iBdStordocDao;

    @Autowired
    private IBdCalbodyDao iBdCalbodyDao;

    @Autowired
    private IBdSalestruDao iBdSalestruDao;

    @Autowired
    private IBdDeptdocDao iBdDeptdocDao;

    @Autowired
    private IBdCumandocDao iBdCumandocDao;

    @Autowired
    private IBdInvmandocDao iBdInvmandocDao;

    @Autowired
    private IBdInvbasdocDao iBdInvbasdocDao;

    @Autowired
    private IBdTaxitemsDao iBdTaxitemsDao;

    @Autowired
    private IBdCubasdocDao iBdCubasdocDao;

    @Autowired
    private IBdDefdocDao iBdDefdocDao;

    @Autowired
    private IBdRdclDao iBdRdclDao;

    /**
     * 初始化公司档案-表头公司
     * 2024年7月31日 11:16:23
     */
//    public static Map<String, BdCorpEntity> stringBdCorpEntityMap = new HashMap<>();
    private Map<String, BdCorpEntity> initShop() throws Exception {
        Map<String, BdCorpEntity> stringBdCorpEntityMap = new HashMap<>();

        BdCorpEntity bdCorpEntity = new BdCorpEntity();
        bdCorpEntity.setDr(0);
        bdCorpEntity.setDataSourceCode("lets_u8c");
        List<BdCorpEntity> bdCorpEntityList = iBdCorpDao.query(bdCorpEntity);
        if (bdCorpEntityList != null && bdCorpEntityList.size() > 0) {
            for (int i = 0; i < bdCorpEntityList.size(); i++) {
                BdCorpEntity bdCorpEntity1 = bdCorpEntityList.get(i);
                stringBdCorpEntityMap.put(bdCorpEntity1.getUnitcode(), bdCorpEntity1);
            }
        }
        return stringBdCorpEntityMap;
    }

    /**
     * 初始化发货仓库
     * 2024年7月31日 11:16:27
     * 202403011513：仓库根据仓库编码+库存组织编码去查询，之前是仓库编码+公司主键，和李佳妮确认好了
     */
//    public static Map<String, BdStordocEntity> stringBdStordocEntityMap = new HashMap<>();
    private Map<String, BdStordocEntity> initBdStordoc() throws Exception {
        Map<String, BdStordocEntity> stringBdStordocEntityMap = new HashMap<>();

        BdStordocEntity bdStordocEntity = new BdStordocEntity();
        bdStordocEntity.setDr(0L);
        bdStordocEntity.setDataSourceCode("lets_u8c");
        List<BdStordocEntity> bdStordocEntityList = iBdStordocDao.query(bdStordocEntity);
        if (bdStordocEntityList != null && bdStordocEntityList.size() > 0) {
            for (int i = 0; i < bdStordocEntityList.size(); i++) {
                BdStordocEntity bdStordocEntity1 = bdStordocEntityList.get(i);
                stringBdStordocEntityMap.put(bdStordocEntity1.getStorcode() + bdStordocEntity1.getPkCalbody(), bdStordocEntity1);
            }
        }
        return stringBdStordocEntityMap;
    }

    /**
     * 2024年7月31日 11:09:12 查询发货库存组织
     */
//    public static Map<String, BdCalbodyEntity> stringBdCalbodyEntityMap = new HashMap<>();
    private Map<String, BdCalbodyEntity> initBdCalbody() throws Exception {
        Map<String, BdCalbodyEntity> stringBdCalbodyEntityMap = new HashMap<>();

        BdCalbodyEntity bdCalbodyEntity = new BdCalbodyEntity();
        bdCalbodyEntity.setDr(0);
        bdCalbodyEntity.setDataSourceCode("lets_u8c");
        List<BdCalbodyEntity> bdCalbodyEntities = iBdCalbodyDao.query(bdCalbodyEntity);
        if (bdCalbodyEntities != null && bdCalbodyEntities.size() > 0) {
            for (int i = 0; i < bdCalbodyEntities.size(); i++) {
                BdCalbodyEntity bdCalbodyEntity1 = bdCalbodyEntities.get(i);
                stringBdCalbodyEntityMap.put(bdCalbodyEntity1.getPkCorp(), bdCalbodyEntity1);
            }
        }
        return stringBdCalbodyEntityMap;
    }

    /**
     * 初始化销售组织
     */
//    public static Map<String, BdSalestruEntity> stringBdSalestruEntityMap = new HashMap<>();
    private Map<String, BdSalestruEntity> initBdSalestru() throws Exception {
        Map<String, BdSalestruEntity> stringBdSalestruEntityMap = new HashMap<>();

        BdSalestruEntity bdSalestruEntity = new BdSalestruEntity();
        bdSalestruEntity.setDr(0);
        bdSalestruEntity.setDataSourceCode("lets_u8c");
        List<BdSalestruEntity> bdSalestruEntityList = iBdSalestruDao.query(bdSalestruEntity);
        if (bdSalestruEntityList != null && bdSalestruEntityList.size() > 0) {
            for (int i = 0; i < bdSalestruEntityList.size(); i++) {
                BdSalestruEntity bdSalestruEntity1 = bdSalestruEntityList.get(i);
                stringBdSalestruEntityMap.put(bdSalestruEntity1.getVsalestruname(), bdSalestruEntity1);
            }
        }
        return stringBdSalestruEntityMap;
    }

    /**
     * 初始化业务部门
     *
     * @author liuyang
     */
//    public static Map<String, BdDeptdocEntity> stringBdDeptdocEntityMap = new HashMap<>();
    private Map<String, BdDeptdocEntity> initDept() {
        Map<String, BdDeptdocEntity> stringBdDeptdocEntityMap = new HashMap<>();

        BdDeptdocEntity bdDeptdocEntity = new BdDeptdocEntity();
        bdDeptdocEntity.setDataSourceCode("lets_u8c");
        bdDeptdocEntity.setDr(0);
        bdDeptdocEntity.setDeptname("其他部门");
        List<BdDeptdocEntity> bdDeptdocEntityList = iBdDeptdocDao.query(bdDeptdocEntity);
        if (bdDeptdocEntityList != null && bdDeptdocEntityList.size() > 0) {
            for (int i = 0; i < bdDeptdocEntityList.size(); i++) {
                BdDeptdocEntity bdDeptdocEntity1 = bdDeptdocEntityList.get(i);
                stringBdDeptdocEntityMap.put(bdDeptdocEntity1.getPkCorp(), bdDeptdocEntity1);
            }
        }
        return stringBdDeptdocEntityMap;
    }

    /**
     * 初始化客商基本档案
     *
     * @author liuyang
     */
//    public static Map<String, BdCubasdocEntity> stringBdCubasdocEntityHashMap = new HashMap<>();
    private Map<String, BdCubasdocEntity> initBdCubasdoc() {
        Map<String, BdCubasdocEntity> stringBdCubasdocEntityHashMap = new HashMap<>();

        BdCubasdocEntity bdCubasdocEntity = new BdCubasdocEntity();
        bdCubasdocEntity.setDataSourceCode("lets_u8c");
        bdCubasdocEntity.setDr(0L);
        List<BdCubasdocEntity> bdCumandocEntityList = iBdCubasdocDao.query(bdCubasdocEntity);
        if (bdCumandocEntityList != null && bdCumandocEntityList.size() > 0) {
            for (int i = 0; i < bdCumandocEntityList.size(); i++) {
                BdCubasdocEntity bdCubasdocEntity1 = bdCumandocEntityList.get(i);
                if (stringBdCubasdocEntityHashMap.get(bdCubasdocEntity1.getDef1()) == null) {
                    stringBdCubasdocEntityHashMap.put(bdCubasdocEntity1.getDef1(), bdCubasdocEntity1);
                } else {
                    logger.warn("客商档案def1重复，客商编码：{}", bdCubasdocEntity1.getCustcode());
                }
            }
        }
        return stringBdCubasdocEntityHashMap;
    }

    /**
     * 初始化客商管理档案
     *
     * @author liuyang
     */
//    public static Map<String, BdCumandocEntity> stringBdCumandocEntityMap = new HashMap<>();
    private Map<String, BdCumandocEntity> initBdCumandoc() {
        Map<String, BdCumandocEntity> stringBdCumandocEntityMap = new HashMap<>();

        BdCumandocEntity bdCumandocEntity1 = new BdCumandocEntity();
        bdCumandocEntity1.setDataSourceCode("lets_u8c");
        bdCumandocEntity1.setDr(0L);
//        bdCumandocEntity1.setCustflags(ProfilesActiveConstant.CUSTOMER);//客商:2客户
        List<BdCumandocEntity> bdCumandocEntityList = iBdCumandocDao.query(bdCumandocEntity1);
        if (bdCumandocEntityList != null && bdCumandocEntityList.size() > 0) {
            for (int i = 0; i < bdCumandocEntityList.size(); i++) {
                BdCumandocEntity bdCumandocEntity = bdCumandocEntityList.get(i);
                stringBdCumandocEntityMap.put(bdCumandocEntity.getPkCubasdoc() + bdCumandocEntity.getPkCorp(), bdCumandocEntity);
            }
        }
        return stringBdCumandocEntityMap;
    }

    /**
     * 初始化存货管理档案
     */
//    public static Map<String, BdInvmandocEntity> stringBdInvmandocEntityMap = new HashMap<>();
    private Map<String, BdInvmandocEntity> initInventoryFile() throws Exception {
        Map<String, BdInvmandocEntity> stringBdInvmandocEntityMap = new HashMap<>();

        BdInvmandocEntity bdInvmandocEntity = new BdInvmandocEntity();
//        bdInvmandocEntity.setInvcode(copyRowDetailsListVo.getSpec_no());
//        bdInvmandocEntity.setPkCorp(shippingCompanyBdCorpEntity.getPkCorp());
        List<BdInvmandocEntity> bdInvmandocEntity2 = iBdInvmandocDao.queryBdInvmandocByInvcodeList(bdInvmandocEntity);
        if (bdInvmandocEntity2 != null && bdInvmandocEntity2.size() > 0) {
            for (int i = 0; i < bdInvmandocEntity2.size(); i++) {
                BdInvmandocEntity bdInvmandocEntity1 = bdInvmandocEntity2.get(i);
                //2024年8月6日 14:27:45 通过存货编码+公司缓存
                stringBdInvmandocEntityMap.put(bdInvmandocEntity1.getInvcode() + bdInvmandocEntity1.getPkCorp(), bdInvmandocEntity1);
            }
        }
        return stringBdInvmandocEntityMap;
    }

    /**
     * 初始化存货基本档案
     * 2024年8月7日 10:57:55
     */
//    public static Map<String, BdInvbasdocEntity> stringBdInvbasdocEntityHashMap = new HashMap<>();
    private Map<String, BdInvbasdocEntity> initBasicInventoryFile() throws Exception {
        Map<String, BdInvbasdocEntity> stringBdInvbasdocEntityHashMap = new HashMap<>();

        BdInvbasdocEntity bdInvbasdocEntity = new BdInvbasdocEntity();
//        bdInvbasdocEntity.setPk_invmandoc(bdInvmandocEntity1.getPkInvmandoc());
//        bdInvbasdocEntity.setPk_corp(shippingCompanyBdCorpEntity.getPkCorp());
        List<BdInvbasdocEntity> bdInvbasdocEntity2 = iBdInvbasdocDao.queryBdInvbasdocByPkInvmandocV2(bdInvbasdocEntity);
        if (bdInvbasdocEntity2 != null && bdInvbasdocEntity2.size() > 0) {
            for (int i = 0; i < bdInvbasdocEntity2.size(); i++) {
                BdInvbasdocEntity bdInvbasdocEntity1 = bdInvbasdocEntity2.get(i);
                stringBdInvbasdocEntityHashMap.put(bdInvbasdocEntity1.getPk_invmandoc() + bdInvbasdocEntity1.getPk_corp_man(), bdInvbasdocEntity1);
            }
        }
        return stringBdInvbasdocEntityHashMap;
    }

    /**
     * 2024年8月7日 10:59:30
     * 初始化存货税率
     *
     * @author liuyang
     */
//    public static Map<String, BdTaxitemsEntity> stringBdTaxitemsEntityHashMap = new HashMap<>();
    private Map<String, BdTaxitemsEntity> initBdTaxitemsEntity() throws Exception {
        Map<String, BdTaxitemsEntity> stringBdTaxitemsEntityHashMap = new HashMap<>();

        BdTaxitemsEntity bdTaxitemsEntity = new BdTaxitemsEntity();
        List<BdTaxitemsEntity> bdTaxitemsEntityList = iBdTaxitemsDao.queryBdInvbasdocByInvcodeV2(bdTaxitemsEntity);
        if (bdTaxitemsEntityList != null && bdTaxitemsEntityList.size() > 0) {
            for (int i = 0; i < bdTaxitemsEntityList.size(); i++) {
                BdTaxitemsEntity bdTaxitemsEntity1 = bdTaxitemsEntityList.get(i);
                stringBdTaxitemsEntityHashMap.put(bdTaxitemsEntity1.getInvcode(), bdTaxitemsEntity1);
            }
        }

        return stringBdTaxitemsEntityHashMap;
    }

    /**
     * 缓存来源平台逻辑
     *
     * @author liuyang
     */
//    public static Map<String, BdDefdocEntity> stringBdDefdocEntityHashMap = new HashMap<>();
    private Map<String, BdDefdocEntity> initSourcePlatform() {
        Map<String, BdDefdocEntity> stringBdDefdocEntityHashMap = new HashMap<>();

        BdDefdocEntity bdDefdocEntity = new BdDefdocEntity();
        bdDefdocEntity.setPkDefdoclist("0001A210000000000JUD");
        bdDefdocEntity.setDr(0);
        bdDefdocEntity.setDataSourceCode("lets_u8c");
        List<BdDefdocEntity> bdDefdocEntityList = iBdDefdocDao.query(bdDefdocEntity);
        if (bdDefdocEntityList != null && bdDefdocEntityList.size() > 0) {
            for (int i = 0; i < bdDefdocEntityList.size(); i++) {
                BdDefdocEntity bdDefdocEntity1 = bdDefdocEntityList.get(i);
                stringBdDefdocEntityHashMap.put(bdDefdocEntity1.getDoccode(), bdDefdocEntity1);
            }
        }
        return stringBdDefdocEntityHashMap;
    }

    /**
     * 2024年8月14日 15:17:48 缓存U8C店铺自定义档案逻辑
     */
//    public static Map<String, BdDefdocEntity> shopDefdocEntityHashMap = new HashMap<>();
    private Map<String, BdDefdocEntity> initShopDiy() {
        Map<String, BdDefdocEntity> shopDefdocEntityHashMap = new HashMap<>();

        BdDefdocEntity bdDefdocEntity = new BdDefdocEntity();
        bdDefdocEntity.setPkDefdoclist("0001A210000000000XZX");
        bdDefdocEntity.setDr(0);
        bdDefdocEntity.setDataSourceCode("lets_u8c");
        List<BdDefdocEntity> bdDefdocEntityList = iBdDefdocDao.query(bdDefdocEntity);
        if (bdDefdocEntityList != null && bdDefdocEntityList.size() > 0) {
            for (int i = 0; i < bdDefdocEntityList.size(); i++) {
                BdDefdocEntity bdDefdocEntity1 = bdDefdocEntityList.get(i);
                shopDefdocEntityHashMap.put(bdDefdocEntity1.getDoccode(), bdDefdocEntity1);
            }
        }
        return shopDefdocEntityHashMap;
    }

    /**
     * 初始化收发类别
     */
    private Map<String, BdRdclEntity> initBdRdcl() {
        Map<String, BdRdclEntity> stringBdRdclEntityHashMap = new HashMap<>();
        BdRdclEntity bdRdclEntity = new BdRdclEntity();
        bdRdclEntity.setDr(0);
        bdRdclEntity.setDataSourceCode("lets_u8c");
        List<BdRdclEntity> bdRdclEntityList = iBdRdclDao.query(bdRdclEntity);
        if (bdRdclEntityList != null && bdRdclEntityList.size() > 0) {
            for (int i = 0; i < bdRdclEntityList.size(); i++) {
                BdRdclEntity bdRdclEntity1 = bdRdclEntityList.get(i);
                stringBdRdclEntityHashMap.put(bdRdclEntity1.getRdcode(), bdRdclEntity1);
            }
        }
        return stringBdRdclEntityHashMap;
    }

    /**
     * 清理缓存
     *
     * @author liuyang
     */
//    public void clearCache() throws Exception {
//        stringBdCorpEntityMap.clear();
//        stringBdStordocEntityMap.clear();
//        stringBdCalbodyEntityMap.clear();
//        stringBdSalestruEntityMap.clear();
//        stringBdDeptdocEntityMap.clear();
//        stringBdCumandocEntityMap.clear();
//        stringBdInvmandocEntityMap.clear();
//        stringBdInvbasdocEntityHashMap.clear();
//        stringBdTaxitemsEntityHashMap.clear();
//        stringBdCubasdocEntityHashMap.clear();
//        stringBdDefdocEntityHashMap.clear();
//        shopDefdocEntityHashMap.clear();
//    }

    /**
     * 初始化缓存
     *
     * @author liuyang
     */
    public CacheTocMapVo initCache() throws Exception {
//        clearCache();

        long startMillis = System.currentTimeMillis();
        logger.info("初始化缓存开始");
        Map<String, BdCorpEntity> stringBdCorpEntityMap = initShop();
        Map<String, BdStordocEntity> stringBdStordocEntityMap = initBdStordoc();
        Map<String, BdCalbodyEntity> stringBdCalbodyEntityMap = initBdCalbody();
        Map<String, BdSalestruEntity> stringBdSalestruEntityMap = initBdSalestru();
        Map<String, BdDeptdocEntity> stringBdDeptdocEntityMap = initDept();
        Map<String, BdCumandocEntity> stringBdCumandocEntityMap = initBdCumandoc();
        Map<String, BdInvmandocEntity> stringBdInvmandocEntityMap = initInventoryFile();
        Map<String, BdInvbasdocEntity> stringBdInvbasdocEntityMap = initBasicInventoryFile();
        Map<String, BdTaxitemsEntity> stringBdTaxitemsEntityMap = initBdTaxitemsEntity();
        Map<String, BdCubasdocEntity> stringBdCubasdocEntityMap = initBdCubasdoc();
        Map<String, BdDefdocEntity> stringBdDefdocEntityMap = initSourcePlatform();
        Map<String, BdDefdocEntity> stringBdDefdocEntityMap1 = initShopDiy();
        Map<String, BdRdclEntity> stringBdRdclEntityMap = initBdRdcl();
        long endMillis = System.currentTimeMillis();
        logger.info("初始化缓存完成 耗时：{}", (endMillis - startMillis));

        CacheTocMapVo cacheTocMapVo = new CacheTocMapVo();
        cacheTocMapVo.setStringBdCorpEntityMap(stringBdCorpEntityMap);
        cacheTocMapVo.setStringBdStordocEntityMap(stringBdStordocEntityMap);
        cacheTocMapVo.setStringBdCalbodyEntityMap(stringBdCalbodyEntityMap);
        cacheTocMapVo.setStringBdSalestruEntityMap(stringBdSalestruEntityMap);
        cacheTocMapVo.setStringBdDeptdocEntityMap(stringBdDeptdocEntityMap);
        cacheTocMapVo.setStringBdCumandocEntityMap(stringBdCumandocEntityMap);
        cacheTocMapVo.setStringBdInvmandocEntityMap(stringBdInvmandocEntityMap);
        cacheTocMapVo.setStringBdInvbasdocEntityMap(stringBdInvbasdocEntityMap);
        cacheTocMapVo.setStringBdTaxitemsEntityMap(stringBdTaxitemsEntityMap);
        cacheTocMapVo.setStringBdCubasdocEntityMap(stringBdCubasdocEntityMap);
        cacheTocMapVo.setStringBdDefdocPlatformEntityMap(stringBdDefdocEntityMap);
        cacheTocMapVo.setStringBdDefdocShopEntityMap(stringBdDefdocEntityMap1);
        cacheTocMapVo.setStringBdRdclEntityMap(stringBdRdclEntityMap);
        return cacheTocMapVo;
    }
}